
#include "common.hxx" 
#include "ship.hxx"
#include "parameters.hxx"
#include "textdisplay.hxx"
#include "asteroid.hxx"
#include "missile.hxx"
#include <string.h>
/* instantiate objects here */
/****************************/

ship ship1;				// object of class ship.
asteroid ast[MAX_NO_OF_ASTEROIDS];	// an array of objects of class asteroid.
missile missile1;			// object of class missile.
parameters paramObj;			// object of the parameters class.It is used to access the default values or user-defined values
textdisplay textObj;			// object of textdisplay class.


// a struct to store how many asteroids are active on the screen
// 	 how many we can be generated for the this run 
//	( ie twice the no of asteroids we start out with)
// 	and how many we started out with.
struct howManyAsteroids
{
	int no_of_active_asteroids;
	int no_of_asteroids;
	int start_no_of_asteroids;	
}astnos;


/************************************************************/
/* Function to detect whether an asteroid hit the spaceship */
/* returns the asteroid number if it has hit the spaceship  */
/*         -1 if there was no collision                     */
/************************************************************/
int didAsteroidHitShip()
{
	float shipX = ship1.getDeltaX();		// get the ship's position.
	float shipY = ship1.getDeltaY();
	float asteroidX[MAX_NO_OF_ASTEROIDS], asteroidY[MAX_NO_OF_ASTEROIDS], asteroidR[MAX_NO_OF_ASTEROIDS];
	int i;
	for (i = 0; i < astnos.no_of_asteroids; i++)
	{	
		asteroidX[i] = ast[i].getdeltaX();	// get the asteroid's position and radius.
		asteroidY[i] = ast[i].getdeltaY();
		asteroidR[i] = ast[i].getRadius();
	}

	for (i = 0; i < astnos.no_of_asteroids; i++)
	{
		if (ast[i].getKilled()!= TRUE)		// check collisions between ship and asteroids which have not been destroyed
		{
			if ((shipY >= (asteroidY[i] - asteroidR[i])) &&		// check if the ship's centre is within the
	                     (shipY <= (asteroidY[i] + asteroidR[i])))          // collision radius of the asteroid.
				if ((shipX >= (asteroidX[i] - asteroidR[i])) &&
				   (shipX <= (asteroidX[i] + asteroidR[i])))
				{
					ship1.setHitByAsteroid();	// set hitByAsteroid flag 
					return i+1;			// return the asteroid number
				}
		}
	}
	return -1; // No collision so return -1.		
}


/************************************************************/
/* Function to detect whether a missile hit an asteroid     */
/* returns the asteroid number if it was hit  		    */
/*         -1 if there was no collision                     */
/************************************************************/
int didMissileHitAsteroid()
{
	float missileX = missile1.getMdeltaX();		// get the missile's position
	float missileY = missile1.getMdeltaY();
	int i;
	if ((missileX == 0.0) && (missileY == 0.0))
		return -1;

	float asteroidX[MAX_NO_OF_ASTEROIDS], 
	      asteroidY[MAX_NO_OF_ASTEROIDS], 
              asteroidR[MAX_NO_OF_ASTEROIDS];

	for ( i = 0; i < (astnos.no_of_asteroids); i++)
	{	
		asteroidX[i] = ast[i].getdeltaX();	// get the poistion of all the asteroids and their radii	
		asteroidY[i] = ast[i].getdeltaY();
		asteroidR[i] = ast[i].getRadius();
	}
	
	for ( i = 0;i < (astnos.no_of_asteroids); i++)
	{
		if ((missileX >= (asteroidX[i] - asteroidR[i])) &&	// check if the missile strays into the asteroid's
		    (missileX <= (asteroidX[i] + asteroidR[i])) &&	// collision radius.
		    (missileY >= (asteroidY[i] - asteroidR[i])) &&
		    (missileY <= (asteroidY[i] + asteroidR[i])))
		     {
			ast[i].setHitByMissile();			// set the hitByMissile flag.
			return (i+1);					// return the asteroid number
		      }	
	}
	return -1;	// No collision so return -1.	

}

/************************************************************/
/* Function to detect whether a missile hit a wall          */
/* returns the TRUE or FALSE		  		    */
/*         				                    */
/************************************************************/

int didMissileCollideWithWall()
{
	float missileX = missile1.getMdeltaX();		          	// get the missile's position
	float missileY = missile1.getMdeltaY();

	if ((missileX < LEFT_EDGE) || (missileX > RIGHT_EDGE))		// hit the left or right wall
		return TRUE;
	else
		if ((missileY < TOP_EDGE) || (missileY > BOTTOM_EDGE))	// hit the top or bottom wall
			return TRUE;
		else
			return FALSE;					// no collison with a wall

}

/************************************************************/
/* Function to generate a random number for the asteroid    */
/* position				  		    */
/* returns the number			                    */
/************************************************************/

float getRandomNumber()
{
		float astX;
		astX = rand()/100.0;					// use the rand function
		if (astX > RIGHT_EDGE) {				// and manipulate the number
		astX = astX/2.0;					// so that it lies within the
		}							// the window 
		int astXi = (int)astX;
		if ((astXi % 2) == 0)
			astX = astX * (-1);
		return astX;
}
			
/*************************************************************/
/* keyboard						     */	
/*							     */  
/* deal with the keyboard input to the program.  	     */	
/* 							     */	
/*************************************************************/

void keyboard(unsigned char userkey, int x, int y)
{

	// If 'j' is pressed turn the ship in the counterclockwise direction
	if (userkey == 'j')
		ship1.Turn(-paramObj.getShipTurnRate());

	// If 'l' is pressed turn the ship in the clockwise direction
	if (userkey == 'l')
		ship1.Turn(paramObj.getShipTurnRate());

	// If 'k' is pressed - start the motion of the spaceship if the game has just been started
	// otherwise add thrust to the spaceship
	if (userkey == 'k') 
	{
		if (ship1.getFirstTime() == FALSE)
			ship1.setFirstTime();
		ship1.Thrust(paramObj.getShipMaxSpeed());

	}

	// If 'i' is pressed then start shooting a missile	
	// The missile starts out from the centre of the spaceship and shoots out in the direction of the 
	// front of the spaceship. Hence the arguments to setMValues are borrowed from the spaceship.
	if (userkey == 'i'){

		if (ship1.getFirstTime() == FALSE)
			ship1.setFirstTime();

		//get centre of the ship and it's current angle
		missile1.setMValues(ship1.getDeltaX(), ship1.getDeltaY(), - ship1.getAngle());	
		
		// Initialise a flag
		missile1.setMissileFired();

		// Choose missile's colors
		missile1.setColor(paramObj.getMissileColorR(), paramObj.getMissileColorG(), paramObj.getMissileColorB());

		// Draw the missile
		missile1.drawMissile();
	}

}


 
/*************************************************************
 *
 * update
 * 
 * update everything that needs to be updated in the scene - this
 * function should be called via a timer to ensure that updates 
 * happen at a regular rate, independent of the graphics update
 * rate
 *
 *************************************************************/

void update(int value)
    {
   
    /* at the end of this function
       reinstate the timer callback */
    /********************************/

	if (ship1.getFirstTime() == TRUE){				
		ship1.resetHitByAsteroid();
	 	ship1.moveShipX(ship1.getThrustX());
		ship1.moveShipY(ship1.getThrustY());
	}
	int i;

	ast[0].Turn(1.0);	
	for ( i = 1 ; i < astnos.start_no_of_asteroids; i++)
		ast[i].Turn(-ast[i-1].getTurnDir());	
	for ( i = astnos.start_no_of_asteroids; i < astnos.no_of_asteroids;i++)
        	ast[i].Turn(ast[i].getTurnDir());
	for( i = 0 ; i < (astnos.no_of_asteroids);i++) 
	{
		ast[i].moveX(paramObj.getAsteroidSpeed());		// move the asteroid with the speed set in 
		ast[i].moveY(paramObj.getAsteroidSpeed());		// the parameters class
	}

    /* check if the ship collides with an asteroid */
    /* Also increase the counter which displays the*/
    /* number of ships destroyed.		   */
    /***********************************************/
 	int hit_by = didAsteroidHitShip();				// call the function
	if ((hit_by > 0) && (!ship1.getKilled()))			// if there is a collision and the ship 
	{								// is alive, kill it and increment the
		ship1.setKilled();					// display.
		textObj.increaseShipCtr();			
	}
	if ((hit_by == -1) && (ship1.getKilled()))			// if there is no collision and the ship 
		ship1.resetKilled();					// was killed because of a previous collision
									// revive it.

    /* check if the missile collides with an asteroid */
    /**************************************************/
	int asteroid_no = didMissileHitAsteroid();


    glutTimerFunc(5, update, 1);
    } 


/**************************************************************/
/*							      */
/* display the text 					      */
/*							      */
/**************************************************************/
void drawTextAndValues()
{

	// set the text Color.
	textObj.setColor(paramObj.getTextColorR(), paramObj.getTextColorG(), paramObj.getTextColorB());
	char someText[100];			// store the strings to be displayed.

	strcpy(someText,"Ships");		
	textObj.drawText(someText,-190,170);	// print "Ship" on the window at the specified location.
	textObj.drawValue(textObj.getShipCtr(),-120,170); // print the number of ships destroyed.

	strcpy(someText,"Asteroids");		
	textObj.drawText(someText,-190,190);	// print "Asteroid" on the window.
	textObj.drawValue(textObj.getAsteroidCtr(),-120,190);	// print number of asteroids destroyed.	

	if (ship1.getFirstTime() == FALSE)
	{
		strcpy(someText,"Press 'K' to begin");
		textObj.drawText(someText, 10, 10);
	}

}

/**************************************************************/
/*							      */
/*draw the spaceship  					      */
/*							      */
/**************************************************************/
void drawSpaceShip()
{   

	if (ship1.getHitByAsteroid())
	{
		ship1.drawExplosion();		// explode the ship
		ship1.resetDeltaXY();		// position it back to the centre where it started from 
		ship1.resetThrust();		// remove the thrust value
		ship1.resetAngle();		// remove the angle by which the ship was last rotated
	}
	else
	{					// the ship is active so draw it
		ship1.scale(1.0);		// normal scale
		ship1.setColor(paramObj.getShipColorR(), paramObj.getShipColorG(), paramObj.getShipColorB());
		ship1.draw();
	}
}

/**************************************************************/
/*							      */
/* draw the missile  					      */
/*							      */
/**************************************************************/
void drawMissile()
{
	if (missile1.isMissileFired())		
	{
		missile1.changeMValues(paramObj.getMissileSpeed()); // move the missile
		missile1.setColor(paramObj.getMissileColorR(), paramObj.getMissileColorG(), paramObj.getMissileColorB());
		if (didMissileCollideWithWall())		    // check for collision with wall	
			missile1.drawExplosion();		    // show a pretty collision	
		else
			missile1.drawMissile();			    // draw the missile	
	}
}


/**************************************************************/
/*							      */
/* draw the asteroids  					      */
/* return a value to indicate whether all the 		      */
/* asteroids are killed					      */					
/**************************************************************/
int drawAsteroids()
{
	for (int i=0; i < astnos.no_of_asteroids; i++)
	{
		if(ast[i].getKilled() != TRUE)			// draw only asteroids that are not yet killed.	
		{
			if (ast[i].getHitByMissile() == TRUE)	// if ast is hit you can do two things
			{
				if (ast[i].getScaleX() == 0.5)	// check if small asteroid was shot at
				{
					ast[i].drawExplosion();	// draw an explosion to show that small asteroid is destroyed
					ast[i].setKilled();			 //  set asteroid's killed flag to true
					textObj.increaseAsteroidCtr();
					missile1.setMValues(0.0,0.0,0.0);	//make the missile incative now
					missile1.resetMissileFired();		
					astnos.no_of_active_asteroids--;	// decrement the number of live asteroids
					if (astnos.no_of_active_asteroids == 0)	// all asteroids destroyed so 
						return TRUE;			// start again 
				}
				else
				{
					ast[i].drawExplosion();		// draw an explosion to show that big asteroid has split
					textObj.increaseAsteroidCtr();  // Increase the count of asteroids destroyed 
					ast[i].scale(0.5);		// scale the asteroid that was shot into half the original size
					ast[i].resetHitByMissile();	// make it active again by resetting the hitByMissile flag
					ast[i].resetKilled();		// make the asteroid alive again
					missile1.setMValues(0.0,0.0,0.0);	// make the missile inactive now
					missile1.resetMissileFired();		

					astnos.no_of_active_asteroids++;	// increase the no of active asteroids 
					astnos.no_of_asteroids++;		// and also the total number of asteroids
							
					int newIndex = astnos.no_of_asteroids - 1;	// prepare the new asteroid
					if (!ast[newIndex].readInput(paramObj.getAsteroidFileName())) 
					{
						printf("Error reading asteroid\n");
						exit(1);
					}
					ast[newIndex].resetHitByMissile();		// not hit by missile yet
					ast[newIndex].resetKilled();			// not destroyed yet
					ast[newIndex].SetDeltaX(ast[i].getdeltaX() + (1 * ast[i].getRadius()));
					ast[newIndex].SetDeltaY(ast[i].getdeltaY() + (1 * ast[i].getRadius()));
					ast[newIndex].CalculateRadius();		// calculate the radius
					ast[newIndex].setFirstTime();			// initialize flags
					ast[newIndex].scale(0.5);			// scale by half
					ast[newIndex].setTurnDir(-ast[i].getTurnDir()); // turn it in the other direction 
					if (ast[i].getAngle() < 0)
						ast[newIndex].Turn(1.0);
					else
						ast[newIndex].Turn(-1.0);		

				}
			}
			else
				{
				ast[i].setColor(paramObj.getAsteroidColorR(), paramObj.getAsteroidColorG(), paramObj.getAsteroidColorB());
				ast[i].draw();	
				}
		}
	}
	return FALSE;
}


/*************************************************************
 *
 * display
 * 
 * this function is called each time the screen is to be drawn
 *************************************************************/
void display(void)
{

    /* clear the screen to the clear colour */
    /****************************************/

    glClear(GL_COLOR_BUFFER_BIT);

    /* draw everything */
    /*******************/
	int start_again=FALSE;			// flag to indicate that all the asteroids are now destroyed
						// so start drawing all the asteroids again.
	drawTextAndValues();	
	drawSpaceShip();
	drawMissile();
	start_again=drawAsteroids();

	if (start_again)
	{
		astnos.no_of_active_asteroids = paramObj.getNumberOfAsteroids();
		astnos.no_of_asteroids = paramObj.getNumberOfAsteroids();
		for (int i = 0; i < astnos.no_of_asteroids; i++)
		{
			ast[i].resetHitByMissile();
			ast[i].resetKilled();
			ast[i].scale(1.0);
		}
		start_again = FALSE;
	}

    /* swap buffers */
    /****************/
  
    glutSwapBuffers();
}

/*************************************************************
 *
 * dealWithParameters
 * 
 * code to deal with the command-line
 * parameters for the program
 *************************************************************/
 
void dealWithParameters(int argc, char **argv)
    {

    for (int i = 1;i < argc;i++)
    {
 	if ((strcmp(argv[i],"-sf") == 0))		// -sf set ship file name
		paramObj.setShipFileName(argv[i+1]);
	if ((strcmp(argv[i],"-af") == 0))		// -af set asteroid file name
		paramObj.setAsteroidFileName(argv[i+1]);
	if ((strcmp(argv[i],"-sc") == 0))		// -sc set ship color
		paramObj.setShipColor(atof(argv[i+1]), atof(argv[i+2]), atof(argv[i+3]));
	if ((strcmp(argv[i],"-ac") == 0))		// -ac set asteroid color
		paramObj.setAsteroidColor(atof(argv[i+1]), atof(argv[i+2]), atof(argv[i+3]));
	if ((strcmp(argv[i],"-mc") == 0))		// -mc set missile color
		paramObj.setMissileColor(atof(argv[i+1]), atof(argv[i+2]), atof(argv[i+3]));
	if ((strcmp(argv[i],"-tc") == 0))		// -tc set text color
		paramObj.setTextColor(atof(argv[i+1]), atof(argv[i+2]), atof(argv[i+3]));
	if ((strcmp(argv[i],"-as") == 0))		// -as set asteroid speed
		paramObj.setAsteroidSpeed(atof(argv[i+1]));
	if ((strcmp(argv[i],"-ss") == 0))		// -ms set ship's maximum speed
		paramObj.setShipMaxSpeed(atof(argv[i+1]));
	if ((strcmp(argv[i],"-tr") == 0))		// -tr set ship's turn rate
		paramObj.setShipTurnRate(atof(argv[i+1]));
	if ((strcmp(argv[i],"-ms") == 0))		// -ms set missile speed
		paramObj.setMissileSpeed(atof(argv[i+1]));
	if ((strcmp(argv[i],"-na") == 0))		// -na set number of asteroids.
		paramObj.setNumberOfAsteroids(atoi(argv[i+1]));

    }		
}

/*************************************************************
 *
 * main
 * 
 *************************************************************/

int main(int argc, char **argv)
    {

    /* deal with any GLUT command Line options */
    /*******************************************/
        
    glutInit(&argc, argv);
    
    /* deal with any command line options for this program */
    /*******************************************************/

    dealWithParameters(argc, argv);

    /* Read in the vertices of the ship	& calculate it's radius */
    /************************************************************/
	if (!ship1.readInput(paramObj.getShipFileName())) 
	{
		printf("Error reading ship\n");
		exit(1);
	}
	ship1.CalculateRadius();
	ship1.SetDeltaX(0.0);		// position the ship in the centre.
	ship1.SetDeltaY(0.0);

	srand(time(NULL));		// seed the rand function.

	astnos.no_of_asteroids = astnos.no_of_active_asteroids
			       = astnos.start_no_of_asteroids
			       = paramObj.getNumberOfAsteroids();	// initialise asteroid nos to the set value

    /* Read in the vertices of the asteroid& calculate radius */
    /************************************************************/

	for (int i = 0;i < astnos.no_of_asteroids;i++)
	{
		if (!ast[i].readInput(paramObj.getAsteroidFileName())) 
		{
			printf("Error reading asteroid\n");
			exit(1);
		}
		ast[i].SetDeltaX(getRandomNumber());	// choose a random position for the asteroid		
		ast[i].SetDeltaY(getRandomNumber());
		ast[i].CalculateRadius();		// calculate the radius
		ast[i].setFirstTime();			// perform initial operations of setting direction flags.

	}
	
    /* create an output window */
    /***************************/

    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize(500, 500);
    glutCreateWindow("Astaroidz");
    if (glutGet(GLUT_WINDOW_COLORMAP_SIZE) != 0)
	{
	printf("FAIL: bad RGBA colormap size\n");
	exit(1);
	}

    /* set up for accepting keyboard commands */
    /******************************************/

    glutKeyboardFunc(keyboard);

    /* set up the logical graphics space */
    /*************************************/
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(LEFT_EDGE, RIGHT_EDGE, BOTTOM_EDGE, TOP_EDGE);
    glClearColor(0.0, 0.0, 0.0, 1.0);

    /* assign the display function */
    /*******************************/

    glutDisplayFunc(display);

    /* assign the idle function */
    /****************************/

    glutIdleFunc(display);

    /* set up a timer to regularly update the scene */
    /************************************************/

    glutTimerFunc(5, update, 1);

    /* set everything going */
    /************************/
    
    glutMainLoop();
  
  return 0;  

}

